package Fb::Collection;

use 5.006;
use strict;
use warnings;
use Fb::Out qw(out);

sub new {
	my ($package,%options)=@_;
	if ( ! $options{objects} ) {
		out('e',"Fb::Collection: No objects given");
		return undef;
	}
	my $me={
		_by => {
			'id/unique' => $options{objects},
		},
	};
	my $self=bless($me,$package);
	return $self;
}

sub byUnique {
	my $self=$_[0];
	return $self->{_by}->{'id/unique'};
}

sub _byify {
	my ($self,$attribute)=@_;
	if ( exists($self->{_by}->{$attribute}) ) {
		return 1;
	}
	my %hash;
	for my $obj (values %{$self->{_by_unique}}) {
		push(@{$hash{$obj->get($attribute)}},$obj);
	}
	$self->{_by}->{$attribute}=\%hash;
	return 1;
}

sub _clearCaches {
	my ($self)=@_;
	for my $cache (keys %{$self->{_by}}) {
		if ( $cache ne 'id/unique' ) {
			delete $self->{_by}->{$cache};
		}
	}
	return 1;
}

sub add {
	my ($self,$object)=@_;
	# We could be more efficient...
	$self->_clearCaches();
	if ( ref($object) eq ref($self) ) {
		for my $unique (keys %{$object->{_by}->{'id/unique'}}) {
			$self->{_by}->{'id/unique'}->{$unique}=$object->{_by}->{'id/unique'}->{$unique};
		}
	} else {
		$self->{_by}->{'id/unique'}->{$object->get('id/unique')}=$object;
	}
	return 1;
}

sub remove {
	my ($self,$object)=@_;
	# We could be more efficient...
	$self->_clearCaches();
	if ( ref($object) eq '' ) {
		delete $self->{_by}->{'id/unique'}->{$object};
	} else {
		delete $self->{_by}->{'id/unique'}->{$object->get('id/unique')};
	}
	return 1;
}

sub iterate {
	my $self=$_[0];
	return values %{$self->{_by}->{'id/unique'}};
}

sub any {
	my $self=$_[0];
	my @keys=keys %{$self->{_by}->{'id/unique'}};
	if ( @keys ) {
		return $self->{_by}->{'id/unique'}->{$keys[0]};
	} else {
		return undef;
	}
}

sub sorted {
	my ($self,$attribute)=@_;
	if ( ! defined($attribute) ) {
		$attribute='id/unique';
	}
	if ( $attribute eq 'id/unique' ) {
		return map { $self->{_by}->{$attribute}->{$_} } sort keys %{$self->{_by}->{$attribute}};
	} else {
		$self->_byify($attribute) || return undef;
		return map { @{$self->{_by}->{$attribute}->{$_}} } sort keys %{$self->{_by}->{$attribute}};
	}
}

=item $object=$collection->find($attribute,$value)

Returns any first object with property I<$attribute> matching I<$value>.

=cut

sub find {
	my ($self,$attribute,$value)=@_;
	$self->_byify($attribute) || return undef;
	my $result=$self->{_by}->{$attribute}->{$value} || return undef;
	return $result->[0];
}

=item \@objects=$collection->filter($attribute,$value)

Returns all objects with scalar property I<$attribute> matching I<$value>.

=cut

sub filtered {
	my ($self,$attribute,$value)=@_;
	$self->_byify($attribute) || return undef;
	my $result=$self->{$attribute}->{$value} || return undef;
	return $result;
}

sub filter {
	my ($self,$constraint)=@_;
	$self->_clearCaches();
	for my $unique (keys %{$self->{_by}->{'id/unique'}}) {
		if ( ! &$constraint($self->{_by}->{$unique}) ) {
			delete($self->{_by}->{$unique});
		}
	}
	return 1;
}

sub count {
	return scalar(keys %{$_[0]->{_by}->{'id/unique'}});
}

sub in {
	out('e',"Collection->in() doesn't exist. Internal error.");
	exit 1;
}

sub contains {
	my ($self,$object)=@_;
	if ( ref($object) eq '' ) {
		return int(exists($self->{_by}->{'id/unique'}->{$object}));
	} else {
		return int(exists($self->{_by}->{'id/unique'}->{$object->get('id/unique')}));
	}
}

sub listformat {
	my ($self,$format)=@_;
	if ( ! defined($format) || $format eq 'collection' ) {
		return $self
	} elsif ( $format eq 'hashref' ) {
		return $self->{_by}->{'id/unique'};
	} elsif ( $format eq 'arrayref' ) {
		# return $self->iterate();
		return [values %{$self->{_by}->{'id/unique'}}];
	} else {
		out('e',"listformat(): Invalid format '$format'");
		return undef;
	}
}

1;
