package Fb::Network;

use 5.006;
use strict;
use warnings;
use Fb::Out qw(out);
use Socket;

require Exporter;
our @ISA=qw(Exporter);

our @EXPORT_OK=qw(
	getFQDN
	getFirstIP
	getLocalFQDN
	prepareFtests
);

our $fake_hostname;
our $fake_getHost;

=over 4

=item $fqdn=getFQDN($host)

Returns the FQDN of host I<$host> or B<undef> if I<$host> is
unresolvable. I<$host> can be either a simple hostname, an actual FQDN or
an IP.

=cut

sub getFQDN {
	my $rawname=shift;
	my @f;
	if ( defined($fake_getHost) ) {
		@f=&{$fake_getHost}($rawname);
		if ( ! @f ) {
			out('d',"\$fake_getHost($rawname): unable to resolv hostname.");
			return undef;
		}
	} else {
		if ( $rawname =~ /^[\.0-9]+$/ ) {
			my $ia=inet_aton($rawname) || return undef;
			@f=gethostbyaddr($ia,2);
			if ( ! @f ) {
				out('d',"gethostbyaddr($rawname): unable to resolv hostname.");
				return undef;
			}
		} else {
			@f=gethostbyname($rawname);
			if ( ! @f ) {
				out('d',"gethostbyname($rawname): unable to resolv hostname.");
				return undef;
			}
		}
	}
	my ($name, $aliases, $addrtype, $length, @addrs)=@f;
	if ( $addrtype != 2 ) {
		out('d',"gethostbyname($rawname): addrtype not AF_INET.");
		return undef;
	}
	if ( @addrs > 1 ) {
		out('d',"gethostbyname($rawname): Multiple IP addresses found.");
		return undef;
	}
	return $name;
}

sub getFirstIP {
	my ($name)=@_;
	my $ip;
	if ( defined($fake_getHost) ) {
		my @f=&{$fake_getHost}($name);
		if ( ! @f ) {
			out('d',"\$fake_getHost($name): unable to resolv hostname.");
			return undef;
		}
		$ip=$f[4];
	} else {
		my @f=gethostbyname($name);
		if ( ! @f ) {
			out('d',"gethostbyname($name): unable to resolv hostname.");
			return undef;
		}
		my @octets=unpack('C4',$f[4]);
		$ip=join('.',@octets);
	}
	return $ip;
}


=item $name=getLocalFQDN();

Returns the local host's FQDN or B<undef> if the FQDN can't be resolved.

=cut

sub getLocalFQDN {
	my $hostname;
	if ( defined($fake_hostname) ) {
		$hostname=$fake_hostname;
	} else {
		$hostname=`hostname`;
		chomp($hostname);
	}
	return getFQDN($hostname);
}

sub prepareFtests {
	my %options=@_;
	if ( $options{hostname} ) {
		$fake_hostname=$options{hostname};
	}
	if ( $options{getHost} ) {
		$fake_getHost=$options{getHost};
	}
	return 1;
}


1;
